<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services\ParamGeraisService;

class ParaGeral extends Crud {

	protected $salesMessage;
    protected $ParaVendaGeralService;
    const DATA_SOURCE_NAME_GERAL = 'vnd10003_paramGerais_Geral';
    const DATA_SOURCE_NAME_FISCAL = 'vnd10003_paramGerais_Fiscal';
    const DATA_SOURCE_NAME_CONTRATO = 'vnd10003_paramGerais_Contrato';
    const DATA_SOURCE_VENDEDOR = 'vnd10003_paramGerais_Delivery_Vendedor';
    
    protected $dataSourceName;

	public function __construct(Utilities\SalesMessage $salesMessage,  Manager $dataSourceManager, Utilities\Util $util, \Doctrine\ORM\EntityManager $entityManager,
	                             ParamGeraisService $paramGeraisService ) {
		$this->salesMessage = $salesMessage;
        parent::__construct($dataSourceManager);
        $this->util          = $util;
        $this->entityManager = $entityManager;
        $this->ParamGeraisService = $paramGeraisService;
	}

    public function findGeral(Request\Filter $request, Response $response){
        $this->dataSourceName = self::DATA_SOURCE_NAME_GERAL;
        parent::find($request,$response);
    }
    
    public function findFiscal(Request\Filter $request, Response $response){
        $this->dataSourceName = self::DATA_SOURCE_NAME_FISCAL;
        parent::find($request,$response);
    }
    
    public function findContrato(Request\Filter $request, Response $response){
        $params = $this->util->getParamsAsArray($request);
        $result = $this->ParamGeraisService->getContrato($params);
        $response->addDataSet(new DataSet("getParaGeraisContrato", $result));
        return $response;
        
    }
    
    public function getVendedor (Request\Filter $request, Response $response){
        $this->dataSourceName = self::DATA_SOURCE_VENDEDOR;
        parent::find($request,$response);
    }
    
    public function saveGeral (Request\DataSet $request, Response $response){
        $this->dataSourceName = self::DATA_SOURCE_NAME_GERAL;
        $this->saveComum($request,$response);
    }
    
    public function saveFiscal (Request\DataSet $request, Response $response){
        $this->dataSourceName = self::DATA_SOURCE_NAME_FISCAL;
        $this->saveComum($request,$response);
    }
    
    private function saveComum (Request\DataSet $request, Response $response){
        try{
            $this->save($request,$response);
       }catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
       }
    }
    
    public function save(Request\DataSet $request, Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    
    public function saveContrato(Request\DataSet $request, Response $response){
        $this->dataSourceName = self::DATA_SOURCE_NAME_CONTRATO;
        $params = $this->util->getParamsAsArray($request);
        $nrContrato = $this->getNewSeqCode($params);
        $params[0]['CDCONTRATOGERAL'] = $nrContrato;
        $this->ParamGeraisService->saveContrato($params);
        
    }
	
	private function getNewSeqCode ($params){
		$params = array("NRORG" => $params[0]['NRORG']);
		$seqcode = $this->entityManager->getConnection()->fetchAll(self::GET_MAX_CDCONTRATO, $params)[0]['MAX'];
		$seqcode++;
		$seqcode = str_pad($seqcode, 4, "0", STR_PAD_LEFT);
		return $seqcode;
	}
	
	const GET_MAX_CDCONTRATO = "
	    SELECT NVL(MAX(CDCONTRATOGERAL), 0) AS MAX FROM CONTRATOGERAL WHERE NRORG = :NRORG
	";
}