<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO\Response\Error;

class TransfSaldo extends CrudWithParameterBag {
    protected $parameterBag;
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, Utilities\Util $util, Services\TransfSaldoService $TransfSaldoService, Services\NovoCodigo $NovoCodigo, $crudService){
        $this->util = $util;
        $this->crudService = $crudService;
        $this->transfSaldoService = $TransfSaldoService;
        $this->novoCodigo = $NovoCodigo;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    
    protected $dataSourceNameArray = array (
        'familiaSaldo'   => 'vnd09500_familiaSaldo',
        'familia'               => 'vnd09500_familia'
    );
    protected $parameterBagColumnsArray = array (
        'familiaSaldo' => array('CDFAMILISALD', 'TCDFAMILISALD', 'NRORG'),
        'familia'              => array('NRORG'),
    );  
    
    public function findFamilia (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['familiaSaldo'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['familiaSaldo'];
        parent::find($request, $response);
    }
    
    public function findAllFamilias (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['familia'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['familia'];
        parent::find($request, $response);
    }
    
    public function transfere(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $saldo = $this->adqSaldoCons($row);
        
        if(!isset($saldo[0]['VRSALDCONFAM'])) {
            print 'Família não cadastrada no consumidor de origem!';
        } else {
            
            //parametros destino
            $paramsDestino = clone $row;
            $paramsDestino['CDCONSUMIDOR'] = $row['CDCONSUMIDOR_DESTINO'];
            $paramsDestino['CDFAMILISALD'] = $row['CDFAMILISALD_DESTINO'];
            unset($paramsDestino['CDCONSUMIDOR_DESTINO']);
            unset($paramsDestino['CDFAMILISALD_DESTINO']);
            
            //verifica saldos
            $saldoDestino = $this->adqSaldoCons($paramsDestino);
        
            if(!isset($saldoDestino[0]['VRSALDCONFAM'])) {
                print 'Família não cadastrada no consumidor de destino!';
            } else {
                if((float)$row['VALOR'] <= 0) {
                    print 'O valor a ser transferido deve ser maior que 0!';
                } else {
                    $saldo = $saldo[0]['VRSALDCONFAM'];
                    
                    if((float)$row['VALOR'] > $saldo) {
                        print 'Saldo insuficiente!' . '<br><br>' . 'Saldo:' . $saldo;
                    } else {
                        
                        //origem
                        $this->setExtrato($row, 'Débito Transf. interna', (float)$row['VALOR'], null, 'D');
                        
                        //destino
                        $this->setExtrato($paramsDestino, 'Crédito Transf. interna', (float)$row['VALOR'], null, 'C');
                        
                        print 'Transferência realizada com sucesso!'
                        .
                        '<br><br>'
                        .
                        'De ' . $row['NMCONSUMIDOR'] . ' - Família ' . $row['NMFAMILISALD'] 
                        .
                        '<br>'
                        .
                        'Para ' . $row['NMCONSUMIDOR_DESTINO'] . ' - Família ' . $row['NMFAMILISALD_DESTINO']
                        ;
                    }
                }
            }
        }
    }
    
    private function setExtrato($row, $msg, $valor, $VRSALDCONEXT, $IDTPMOVEXT) {
        $saldoAtual = $this->adqSaldoCons($row);
        $saldoAtual = $saldoAtual[0]['VRSALDCONFAM'];
        $cdsSeqMovExtr = $this->adqSeqMovExtr($row);
        $CDTIPORECE = $this->adqTipoRece($row);
        $this->acmInsMovExtrato($row, $cdsSeqMovExtr[0]['NRSEQMOVEXT'], $CDTIPORECE[0]['CDTIPORECE'], $msg, $valor, $VRSALDCONEXT, $IDTPMOVEXT);
        
        if($IDTPMOVEXT == 'C') {
            $novoSaldo = $saldoAtual+$valor;
        } else {
            $novoSaldo = $saldoAtual-$valor;
        }
        
        $this->acmUpdate($row, $novoSaldo);
    }
    
    ///queries
    private function adqSaldoCons($row) {
        $params = array(
            ':CDCLIENTE'    => $row['CDCLIENTE'],
            ':CDCONSUMIDOR' => $row['CDCONSUMIDOR'],
            ':CDFAMILISALD' => $row['CDFAMILISALD'],
            ':NRORG'        => (string)$row['NRORG']
        );
        
        $resultadoService = $this->transfSaldoService->adqSaldoCons($params);
        return $resultadoService;
    }
    
    private function adqSeqMovExtr($row) {
        $params = array(
            ':CDCLIENTE'    => $row['CDCLIENTE'],
            ':CDCONSUMIDOR' => $row['CDCONSUMIDOR'],
            ':CDFAMILISALD' => $row['CDFAMILISALD'],
            ':DTMOVEXTCONS' => $row['DTMOVEXTCONS'],
            ':NRORG'        => $row['NRORG']
        );
        
        $resultadoService = $this->transfSaldoService->adqSeqMovExtr($params);
        return $resultadoService;
    }
    
    private function adqTipoRece($row) {
        $params = array(
            ':NRORG' => $row['NRORG']
        );
        
        $resultadoService = $this->transfSaldoService->adqTipoRece($params);
        return $resultadoService;
    }
    
    private function acmInsMovExtrato($row, $NRSEQMOVEXT, $CDTIPORECE, $msg, $VRMOVEXTCONS, $VRSALDCONEXT, $IDTPMOVEXT) {
        
        $params = array(
            ':CDCLIENTE'       => $row['CDCLIENTE'],
            ':CDCONSUMIDOR'    => $row['CDCONSUMIDOR'],
            ':CDFAMILISALD'    => $row['CDFAMILISALD'],
            ':DTMOVEXTCONS'    => $row['DTMOVEXTCONS'],
            ':NRSEQMOVEXT'     => $NRSEQMOVEXT,
            ':CDTIPORECE'      => $CDTIPORECE,
            ':DSOPEEXTCONS'    => $msg,
            ':VRMOVEXTCONS'    => (float)$VRMOVEXTCONS,
            ':VRSALDCONEXT'    => (float)$VRSALDCONEXT,
            ':IDTPMOVEXT'      => $IDTPMOVEXT,
            ':NRSEQREGCREDVND' => $this->novoCodigo->getCodeByTableName('EXTRATOCONSVND', $row['NRORG'])
        );
        
        $resultadoService = $this->transfSaldoService->acmInsMovExtrato($params);
        return $resultadoService;
    }
    private function acmUpdate($row, $VRSALDCONFAM) {
        
        $params = array(
            ':CDCLIENTE'    => $row['CDCLIENTE'],
            ':CDCONSUMIDOR' => $row['CDCONSUMIDOR'],
            ':CDFAMILISALD' => $row['CDFAMILISALD'],
            ':VRSALDCONFAM' => $VRSALDCONFAM
        );
        
        $resultadoService = $this->transfSaldoService->acmUpdate($params);
        return $resultadoService;
    }
}