<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RecebimentoServices {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        
    }
	
	public function fetchSynthReport($conditions){
		$params = self::buildSynthQueryParams($conditions);
		return $this->entityManager->getConnection()->fetchAll(self::GET_SYNTH_REPORT, $params);
	}
	
	
	private function buildSynthQueryParams($conditions){
		if(array_key_exists('CDFILIAL', $conditions)){
			$conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
			$conditions['TCDFILIAL'] = NULL;
		} else {
			$conditions['CDFILIAL'] = NULL;
			$conditions['TCDFILIAL'] = 'T';
		}
		if(array_key_exists('CDLOJA', $conditions)){
			$conditions['CDLOJA'] = implode('_', $conditions['CDLOJA']);
			$conditions['TCDLOJA'] = NULL;
		} else {
			$conditions['CDLOJA'] = NULL;
			$conditions['TCDLOJA'] = 'T';
		}
		$params = array(
			':CDFILIAL'   => $conditions['CDFILIAL'],
			':TCDFILIAL'  => $conditions['TCDFILIAL'],
			':CDLOJA'     => $conditions['CDLOJA'],
			':TCDLOJA'    => $conditions['TCDLOJA'],
			':DTINI'      => $conditions['DTENTRVENDA'][0],
			':DTFIN'      => $conditions['DTENTRVENDA'][1],
			':NRORG'      => $conditions['NRORG']
		);
		return $params;
	}
	public function buscaRecebimento($cdfilial, $cdloja, $dtinicial,$dtfinal,$nrorg) {
	
 		$rowResult = [];
		$params = array(
	     	":P_CDFILIAL" => (($cdfilial) != 0) ? implode('_' ,$cdfilial) : null,
	     	":P_CDLOJA" => is_array($cdloja) ? implode('_' ,$cdloja) : null,
	     	":P_TCDLOJA" => is_array($cdloja) ?  NULL : "T",
	     	":P_TCDFILIAL" => is_array($cdfilial) ?  NULL : "T",
	      	"P_DTINI" => $dtinicial,
	     	"P_DTFIN" => $dtfinal,
			"NRORG" => $nrorg
 		);
 		$rowRecebimento = $this->entityManager->getConnection()->fetchAll(self::DADOS_RECEBIMENTO, $params);
 		
 		if(is_array($rowRecebimento)){
	 	 	for($j = 0;  $j < count($rowRecebimento); $j++){
	 	 		
	 	 		$paramBuscaNomeFilial = array(
		     		":P_CDFILIAL" => $rowRecebimento[$j]['CDFILIAL']
	 			);
	 			
		 		$nomeFilial = $this->entityManager->getConnection()->fetchAll(self::BUSCA_NOME_FILIAL, $paramBuscaNomeFilial);
				
				$param_recebimento = array(
			 		":CDFILIAL" => $rowRecebimento[$j]['CDFILIAL'],
				  	"P_DTINI" => $dtinicial,
				 	"P_DTFIN" => $dtfinal,
				 	"NRORG" => $nrorg,
				 	":CDTIPORECE" => $rowRecebimento[$j]["CDTIPORECE"]
		 		);
		 		
		 		$rowTotalSaida = $this->entityManager->getConnection()->fetchAll(self::PER_SAI_RECEBIMENTO, $param_recebimento);
		 		
		 		$queryEntrada  = $this->entityManager->getConnection()->fetchAll(self::ENTR_RECEBIMENTO, $param_recebimento);
		 		
		 		$querySaida    = $this->entityManager->getConnection()->fetchAll(self::SAI_RECEBIMENTO, $param_recebimento);
				
				$queryTotalEntrada  = $this->entityManager->getConnection()->fetchAll(self::PER_ENTR_RECEBIMENTO, $param_recebimento);
				
				
				if(!empty($queryEntrada[0]['ENTRADA'])){
					$entrada = $queryEntrada[0]['ENTRADA'];
				} else {
					$entrada = 0.00;
				}
				
		
				if($querySaida){
					$saida = $querySaida[0]['SAIDA'];
					if($querySaida[0]['SAIDA'] == null || $querySaida[0]['SAIDA'] == 0){
						$saida = $querySaida[0]['SAIDA'] = 0.00;
					}
					
				}else{
					$saida = 0.00;
				}
				
				$valortotal = $entrada - $saida;
				$total = $rowRecebimento[$j]['ENTRADATOT'];
				
				$rowResult[$j]['ENTRADA'] = $entrada;
			 	$rowResult[$j]['SAIDA'] = $saida;
				$rowResult[$j]['TOTAL'] = $valortotal;
			    $rowResult[$j]['CDFILIAL'] =  $rowRecebimento[$j]['CDFILIAL'];
			    $rowResult[$j]['NMFILIAL'] =  ($nomeFilial[0]['NMFILIAL']);
			    $rowResult[$j]['NMTIPORECEC'] = $rowRecebimento[$j]['NMTIPORECE'];
			    if($total !== 0){
			    	$rowResult[$j]['PERCTOT'] = ($valortotal/$total)*100;
			    }else{
			    	$rowResult [$j]['PERCTOT'] = 0.00;
			    }
		    }
 		}
	     
	    return $rowResult;
	}
	
	const DADOS_RECEBIMENTO = "
		   SELECT DISTINCT DADOSCOMSAIDA.CDFILIAL, DADOSCOMSAIDA.NMFILIAL, DADOSCOMSAIDA.CDTIPORECE, DADOSCOMSAIDA.NMTIPORECE,       
		          SUM(NVL(ENT.VRMOVIVEND,0)) OVER (PARTITION  BY DADOSCOMSAIDA.CDFILIAL, DADOSCOMSAIDA.CDTIPORECE) ENTRADA,
		          SUM(NVL(ENT.VRMOVIVEND,0)) OVER (PARTITION  BY DADOSCOMSAIDA.CDFILIAL) ENTRADATOT,
		          DADOSCOMSAIDA.SAIDA, DADOSCOMSAIDA.SAIDATOT
			 FROM (SELECT DISTINCT DADOSVENDA.CDFILIAL, DADOSVENDA.NMFILIAL, DADOSVENDA.CDTIPORECE, DADOSVENDA.NMTIPORECE,
			              SUM(NVL(SAID.VRMOVIVEND,0)) OVER (PARTITION  BY DADOSVENDA.CDFILIAL, DADOSVENDA.CDTIPORECE) SAIDA,          
			              SUM(NVL(SAID.VRMOVIVEND,0)) OVER (PARTITION  BY DADOSVENDA.CDFILIAL) SAIDATOT,
			              DADOSVENDA.DTINICIAL, DADOSVENDA.DTFINAL
			         FROM (SELECT DISTINCT DADOSVENDA.CDFILIAL, DADOSVENDA.NMFILIAL,  T.CDTIPORECE, T.NMTIPORECE, DADOSVENDA.CDCAIXA,
			                      C.NMCAIXA, DADOSVENDA.NRSEQVENDA, :P_DTINI DTINICIAL, :P_DTFIN DTFINAL
			                 FROM (SELECT DISTINCT AUX.CDFILIAL, AUX.NMFILIAL, AUX.CDLOJA, AUX.NMLOJA, V.CDCAIXA,
			                              V.NRSEQVENDA
			                         FROM (SELECT F.CDFILIAL, F.NMFILIAL, L.CDLOJA, L.NMLOJA, L.CHAVELOJA
			                                 FROM (SELECT CDFILIAL, NMFILIAL
			                                         FROM FILIAL
			                                        WHERE (INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T')
			                                          AND NRORG = :NRORG) F,
			                                      		(SELECT DISTINCT CDFILIAL, CDLOJA, NMLOJA, CDFILIAL||CDLOJA CHAVELOJA, NRORG
			                                      		   FROM LOJA)L
			                                WHERE L.CDFILIAL = F.CDFILIAL
			                                  AND ((INSTR(:P_CDLOJA, L.CHAVELOJA) <> 0) OR (:P_TCDLOJA   = 'T'))
			                                  AND L.NRORG = :NRORG) AUX, VENDA V
			                        WHERE AUX.CDFILIAL = V.CDFILIAL
			                          AND AUX.CDLOJA = V.CDLOJA
			                          AND V.DTENTRVENDA BETWEEN :P_DTINI AND :P_DTFIN
			                          AND V.IDSITUVENDA = 'O') DADOSVENDA, CAIXA C, MOVCAIXA M, TIPORECE T
			                WHERE DADOSVENDA.CDFILIAL = C.CDFILIAL
			                  AND DADOSVENDA.CDCAIXA = C.CDCAIXA
			                  AND DADOSVENDA.CDFILIAL = M.CDFILIAL
			                  AND DADOSVENDA.CDCAIXA = M.CDCAIXA
			                  AND DADOSVENDA.NRSEQVENDA = M.NRSEQVENDA
			                  AND M.CDTIPORECE = T.CDTIPORECE) DADOSVENDA 
			         LEFT JOIN MOVCAIXA SAID
			           ON DADOSVENDA.CDFILIAL = SAID.CDFILIAL
			          AND DADOSVENDA.CDCAIXA = SAID.CDCAIXA
			          AND DADOSVENDA.NRSEQVENDA = SAID.NRSEQVENDA
			          AND DADOSVENDA.CDTIPORECE = SAID.CDTIPORECE
			          AND SAID.IDTIPOMOVIVE = 'S' ) DADOSCOMSAIDA 
		     LEFT JOIN MOVCAIXA ENT
		       ON DADOSCOMSAIDA.CDFILIAL = ENT.CDFILIAL
		      AND ENT.DTMOVIMCAIXA BETWEEN DADOSCOMSAIDA.DTINICIAL AND DADOSCOMSAIDA.DTFINAL
		      AND DADOSCOMSAIDA.CDTIPORECE = ENT.CDTIPORECE
		      AND ENT.IDTIPOMOVIVE IN ('E', 'T')
			ORDER BY DADOSCOMSAIDA.CDFILIAL, DADOSCOMSAIDA.CDTIPORECE
		";   
	const BUSCA_NOME_FILIAL ="
		SELECT CDFILIAL, CDFILIAL || ' - '|| NMFILIAL AS NMFILIAL
		   FROM FILIAL
		 WHERE CDFILIAL = :P_CDFILIAL
	";
		
	const PER_SAI_RECEBIMENTO = "
		SELECT SUM(NVL(M.VRMOVIVEND,0)) AS SAIDA
		  FROM VENDA V, MOVCAIXA M
		 WHERE (V.CDFILIAL = M.CDFILIAL ) 
		   AND (V.CDCAIXA = M.CDCAIXA ) 
		   AND (V.NRSEQVENDA = M.NRSEQVENDA) 
		   AND (V.NRORG = M.NRORG) 
		   AND (V.IDSITUVENDA = 'O' ) 
		   AND (V.CDFILIAL = :CDFILIAL ) 
		   AND (V.NRORG = :NRORG) 
		   AND (M.IDTIPOMOVIVE = 'S' ) 
		   AND (V.DTENTRVENDA BETWEEN :P_DTINI AND :P_DTFIN)
	";

	const ENTR_RECEBIMENTO = "
		SELECT SUM(NVL(M.VRMOVIVEND,0)) AS ENTRADA
		  FROM MOVCAIXA M
		 WHERE (M.CDFILIAL     = :CDFILIAL   ) 
		   AND (M.CDTIPORECE   = :CDTIPORECE ) 
		   AND (M.NRSEQVENDA   IS NOT NULL   ) 
		   AND (M.IDTIPOMOVIVE IN ('E', 'T') ) 
		   AND (M.DTMOVIMCAIXA BETWEEN :P_DTINI AND :P_DTFIN)
	";
	
	const SAI_RECEBIMENTO = "
        SELECT SUM(NVL(M.VRMOVIVEND,0)) AS SAIDA
          FROM VENDA V, MOVCAIXA M
         WHERE (V.CDFILIAL = M.CDFILIAL ) 
           AND (V.CDCAIXA = M.CDCAIXA ) 
           AND (V.NRSEQVENDA = M.NRSEQVENDA) 
           AND (V.NRORG = M.NRORG) 
           AND (V.IDSITUVENDA = 'O' ) 
           AND (V.CDFILIAL = :CDFILIAL ) 
           AND (M.CDTIPORECE = :CDTIPORECE ) 
           AND (V.NRORG = :NRORG) 
           AND (M.IDTIPOMOVIVE = 'S' ) 
           AND (V.DTENTRVENDA BETWEEN :P_DTINI AND :P_DTFIN)
    ";
    
    const PER_ENTR_RECEBIMENTO = "
		SELECT SUM(NVL(M.VRMOVIVEND,0)) AS ENTRADA
		  FROM MOVCAIXA M
		 WHERE (M.CDFILIAL     = :CDFILIAL   ) 
		   AND (M.CDTIPORECE   = :CDTIPORECE )
		   AND (M.NRSEQVENDA   IS NOT  NULL  )
		   AND (M.IDTIPOMOVIVE = 'G'         )
		   AND (M.DTMOVIMCAIXA BETWEEN :P_DTINI AND :P_DTFIN)
	";
	
	
	const GET_SYNTH_REPORT = "
		SELECT T.CDTIPORECE, T.NMTIPORECE AS NMTIPORECEC, 'TODOS' AS NMFILIAL, T.ENTRADA, T.SAIDA, T.ENTRADATOT, T.SAIDATOT, T.TOTAL, TRUNC((T.TOTAL * 100 / T.SOMATOT), 2) AS PERCTOT
		FROM (
		      SELECT TOT.CDTIPORECE, TOT.NMTIPORECE, TOT.ENTRADA, TOT.SAIDA, TOT.ENTRADATOT, TOT.SAIDATOT, (TOT.ENTRADA - TOT.SAIDA) AS TOTAL,
		        SUM((TOT.ENTRADA - TOT.SAIDA)) OVER (PARTITION  BY '1') AS SOMATOT
		      FROM (
		            SELECT DISTINCT DADOSVENDA.CDTIPORECE, DADOSVENDA.NMTIPORECE, DADOSVENDA.CDFILIAL,
		              SUM(DECODE(MOV.IDTIPOMOVIVE, 'S', 0, NVL(MOV.VRMOVIVEND,0))) OVER (PARTITION  BY DADOSVENDA.CDTIPORECE) ENTRADA,
		              SUM(DECODE(MOV.IDTIPOMOVIVE, 'S', NVL(MOV.VRMOVIVEND,0), 0)) OVER (PARTITION  BY DADOSVENDA.CDTIPORECE) SAIDA,
		              SUM(DECODE(MOV.IDTIPOMOVIVE, 'S', 0, NVL(MOV.VRMOVIVEND,0))) OVER (PARTITION  BY '1') ENTRADATOT,
		              SUM(DECODE(MOV.IDTIPOMOVIVE, 'S', NVL(MOV.VRMOVIVEND,0), 0)) OVER (PARTITION  BY '1') SAIDATOT
		            FROM (
		                    SELECT DISTINCT DADOSVENDA.CDFILIAL, DADOSVENDA.NMFILIAL, DADOSVENDA.CDLOJA, DADOSVENDA.NMLOJA,
		                      T.CDTIPORECE, T.NMTIPORECE, DADOSVENDA.CDCAIXA, C.NMCAIXA, DADOSVENDA.NRSEQVENDA
		                    FROM (
		                            SELECT DISTINCT AUX.CDFILIAL, AUX.NMFILIAL, AUX.CDLOJA, AUX.NMLOJA, V.CDCAIXA, V.NRSEQVENDA
		                            FROM (
		                                   SELECT F.CDFILIAL, F.NMFILIAL, L.CDLOJA, L.NMLOJA, L.CHAVELOJA
		                                   FROM (
		                                          SELECT CDFILIAL, NMFILIAL
		                                          FROM FILIAL
		                                          WHERE (INSTR(:CDFILIAL, CDFILIAL) <> 0) OR (:TCDFILIAL = 'T')
		                                        ) F,
		                                       (SELECT DISTINCT CDFILIAL, CDLOJA, NMLOJA, CDFILIAL||CDLOJA CHAVELOJA
		                                          FROM LOJA)L
		                                   WHERE L.CDFILIAL = F.CDFILIAL
		                                    AND ((INSTR(:CDLOJA, L.CHAVELOJA) <> 0) OR (:TCDLOJA   = 'T'))
		                                 ) AUX, VENDA V
		                            WHERE AUX.CDFILIAL = V.CDFILIAL
		                              AND AUX.CDLOJA = V.CDLOJA
		                              AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIN                              
		                              AND V.IDSITUVENDA = 'O'
		                              AND V.NRORG = :NRORG
		                         ) DADOSVENDA, CAIXA C, MOVCAIXA M, TIPORECE T
		                    WHERE DADOSVENDA.CDFILIAL = C.CDFILIAL
		                      AND DADOSVENDA.CDCAIXA = C.CDCAIXA
		                      AND DADOSVENDA.CDFILIAL = M.CDFILIAL
		                      AND DADOSVENDA.CDCAIXA = M.CDCAIXA
		                      AND DADOSVENDA.NRSEQVENDA = M.NRSEQVENDA
		                      AND M.CDTIPORECE = T.CDTIPORECE
		                 ) DADOSVENDA 
		            LEFT JOIN MOVCAIXA MOV
		              ON DADOSVENDA.CDFILIAL = MOV.CDFILIAL
		                AND DADOSVENDA.CDCAIXA = MOV.CDCAIXA
		                AND DADOSVENDA.NRSEQVENDA = MOV.NRSEQVENDA
		                AND DADOSVENDA.CDTIPORECE = MOV.CDTIPORECE
		                AND MOV.IDTIPOMOVIVE IN ('T', 'E','S')
		            ORDER BY DADOSVENDA.CDTIPORECE
		           ) TOT
		      GROUP BY TOT.CDTIPORECE, TOT.NMTIPORECE, TOT.ENTRADA, TOT.SAIDA, TOT.ENTRADATOT, TOT.SAIDATOT, (TOT.ENTRADA - TOT.SAIDA)
		      ORDER BY TOT.CDTIPORECE
		     ) T 
		GROUP BY T.CDTIPORECE, T.NMTIPORECE, T.ENTRADA, T.SAIDA, T.ENTRADATOT, T.SAIDATOT, T.TOTAL, TRUNC((T.TOTAL * 100 / T.SOMATOT), 2)
		ORDER BY T.CDTIPORECE
	";
	
}
